#include <libwccl/ops/match/conditions/repeatedmatch.h>
#include <libwccl/values/matchvector.h>
#include <sstream>

namespace Wccl {

RepeatedMatch::RepeatedMatch(const boost::shared_ptr<ConjConditions>& conditions)
	: _conditions(conditions)
{
	BOOST_ASSERT(_conditions);
}

MatchResult RepeatedMatch::apply(boost::shared_ptr<Position> iter_pos, const FunExecContext& context) const
{
	boost::shared_ptr<MatchVector> match(new MatchVector());
	int orig_pos = iter_pos->get_value();
	MatchResult res = _conditions->apply(iter_pos, context);
	if (res.matched()) {
		do {
			match->append(res.get_match());
			res = _conditions->apply(iter_pos, context);
		} while (res.matched());
		return MatchResult(match);
	}
	else {
		iter_pos->set_value(orig_pos);
		return MatchResult();
	}
}

std::string RepeatedMatch::to_string(const Corpus2::Tagset& tagset) const
{
	std::ostringstream ostream;
	ostream << name() << _conditions->to_string(tagset);
	return ostream.str();
}

std::ostream& RepeatedMatch::write_to(std::ostream& ostream) const
{
	return ostream << name() << *_conditions;
}

} /* end ns Wccl */
